from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
from pprint import pprint

from sx126x import SX126X, Mode


def main():
    ap = ArgumentParser(formatter_class=ArgumentDefaultsHelpFormatter)
    ap.add_argument("-p", "--port", default=None, type=str, help="Serial port")
    ap.add_argument("-d", "--debug", action="store_true", help="Enable debug mode")
    args = ap.parse_args()

    lora = SX126X(
        port=args.port,
        debug=args.debug,
        overwrite_defaults=False
    )
    pprint(lora.to_json())


if __name__ == '__main__':
    main()



from enum import Enum


class TransferMethod(Enum):
    TRANSPARENT = 0
    FIXED = 1

    @staticmethod
    def from_data(data: int) -> "TransferMethod":
        return TransferMethod((data >> 6) & 0b1)

    def apply_to(self, data: int) -> int:
        return (data & 0b10111111) | (self.value << 6)



from dataclasses import dataclass


def parse_crypt_key(addr: str) -> tuple[int, int]:
    parts = addr.split(":")
    if len(parts) != 2:
        raise ValueError("Key must be in format 'H:L', (hex)")
    return int(parts[0], 16), int(parts[1], 16)


@dataclass
class CryptKey:
    hi: int
    lo: int

    @staticmethod
    def parse(key: str) -> "CryptKey":
        return CryptKey(*parse_crypt_key(key))

    def __str__(self):
        return f"{self.hi:X}:{self.lo:X}"

    @classmethod
    def from_data(cls, data: bytes) -> "CryptKey":
        if len(data) != 2:
            raise ValueError("Data must be 2 bytes")
        return cls(data[0], data[1])

    def to_bytes(self) -> bytes:
        return bytes([self.hi, self.lo])



from sx126x.sx126x import SX126X
from sx126x.enums import *
from sx126x.models import *

__version__ = "2.0.1"  # Keep in sync with pyproject.toml

__all__ = [
    "SX126X",
    "AirSpeed",
    "AmbientNoise",
    "BaudRate",
    "Command",
    "LBT",
    "Mode",
    "PacketSize",
    "Parity",
    "Register",
    "Relay",
    "RSSI",
    "TransferMethod",
    "TransmitPower",
    "WORControl",
    "WORPeriod",
    "Address",
    "CryptKey",
]



from enum import IntEnum


class PacketSize(IntEnum):
    SIZE_240 = 240
    SIZE_128 = 128
    SIZE_64  = 64
    SIZE_32  = 32

    @staticmethod
    def from_data(data: int) -> "PacketSize":
        bits = (data & 0b11000000) >> 6
        if bits == 0b00:
            return PacketSize.SIZE_240
        elif bits == 0b01:
            return PacketSize.SIZE_128
        elif bits == 0b10:
            return PacketSize.SIZE_64
        elif bits == 0b11:
            return PacketSize.SIZE_32
        else:
            raise ValueError(f"Invalid packet size bits: {bits:02b}")

    def apply_to(self, data: int) -> int:
        if self == PacketSize.SIZE_240:
            bits = 0b00
        elif self == PacketSize.SIZE_128:
            bits = 0b01
        elif self == PacketSize.SIZE_64:
            bits = 0b10
        elif self == PacketSize.SIZE_32:
            bits = 0b11
        else:
            raise ValueError(f"Unsupported packet size: {self}")

        return (data & 0b00111111) | (bits << 6)



from enum import Enum

class AirSpeed(Enum):
    K0_3 = 0.3
    K1_2 = 1.2
    K2_4 = 2.4
    K4_8 = 4.8
    K9_6 = 9.6
    K19_2 = 19.2
    K38_4 = 38.4
    K62_5 = 62.5

    @staticmethod
    def from_data(data: int) -> "AirSpeed":
        bits = data & 0b00000111
        if bits == 0b000:
            return AirSpeed.K0_3
        elif bits == 0b001:
            return AirSpeed.K1_2
        elif bits == 0b010:
            return AirSpeed.K2_4
        elif bits == 0b011:
            return AirSpeed.K4_8
        elif bits == 0b100:
            return AirSpeed.K9_6
        elif bits == 0b101:
            return AirSpeed.K19_2
        elif bits == 0b110:
            return AirSpeed.K38_4
        elif bits == 0b111:
            return AirSpeed.K62_5
        else:
            raise ValueError(f"Invalid air speed bits: {bits:03b}")

    def apply_to(self, data: int) -> int:
        if self == AirSpeed.K0_3:
            bits = 0b000
        elif self == AirSpeed.K1_2:
            bits = 0b001
        elif self == AirSpeed.K2_4:
            bits = 0b010
        elif self == AirSpeed.K4_8:
            bits = 0b011
        elif self == AirSpeed.K9_6:
            bits = 0b100
        elif self == AirSpeed.K19_2:
            bits = 0b101
        elif self == AirSpeed.K38_4:
            bits = 0b110
        elif self == AirSpeed.K62_5:
            bits = 0b111
        else:
            raise ValueError(f"Unsupported air speed: {self}")

        return (data & 0b11111000) | bits



from unittest import TestCase

from sx126x import BaudRate, Address

from sx126x.enums import TransmitPower, RSSI, TransferMethod, Relay, LBT, WORControl, WORPeriod, AirSpeed, AmbientNoise, \
    PacketSize, Mode, Parity
from sx126x.models import CryptKey
from sx126x.sx126x import SX126X


class ConfigurationTests(TestCase):
    def setUp(self):
        self.lora = SX126X(
            port="/dev/ttyAMA0",
            write_persist=True,
            mode=Mode.CONFIGURATION,
            debug=True,
            overwrite_defaults=False,
        )

    def tearDown(self):
        del self.lora

    def test_set_mode(self):
        self.assertEqual(Mode.CONFIGURATION, self.lora.mode)
        self.lora.set_mode(Mode.TRANSMISSION)
        self.assertEqual(Mode.TRANSMISSION, self.lora.mode)
        self.lora.set_mode(Mode.WOR)
        self.assertEqual(Mode.WOR, self.lora.mode)
        self.lora.set_mode(Mode.DEEP_SLEEP)
        self.assertEqual(Mode.DEEP_SLEEP, self.lora.mode)

    def test_address_h(self):
        self.assertEqual(Mode.CONFIGURATION, self.lora.mode)
        self.lora.address_h = 69
        self.assertEqual(69, self.lora.address_h)
        self.lora.address_h = 242
        self.assertEqual(242, self.lora.address_h)

    def test_addr_l(self):
        self.assertEqual(Mode.CONFIGURATION, self.lora.mode)
        self.lora.address_l = 42
        self.assertEqual(42, self.lora.address_l)
        self.lora.address_l = 242
        self.assertEqual(242, self.lora.address_l)

    def test_addr(self):
        self.assertEqual(Mode.CONFIGURATION, self.lora.mode)
        expected_address = Address(242, 69)
        self.lora.address = expected_address
        self.assertEqual(expected_address.hi, self.lora.address_h)
        self.assertEqual(expected_address.lo, self.lora.address_l)
        self.assertEqual(expected_address.__str__(), self.lora.address.__str__())
        self.assertEqual(expected_address, self.lora.address)

    def test_net_id(self):
        self.assertEqual(Mode.CONFIGURATION, self.lora.mode)
        self.lora.net_id = 43
        self.assertEqual(43, self.lora.net_id)

    def test_channel(self):
        self.assertEqual(Mode.CONFIGURATION, self.lora.mode)
        for c in range(0, 50):
            self.lora.channel = c
            self.assertEqual(c, self.lora.channel)

    def test_baud_rate(self):
        self.assertEqual(Mode.CONFIGURATION, self.lora.mode)
        expected_baud_rate = BaudRate.B9600
        self.lora.baud_rate = expected_baud_rate
        self.assertEqual(expected_baud_rate, self.lora.baud_rate)

    def test_parity(self):
        self.assertEqual(Mode.CONFIGURATION, self.lora.mode)
        expected_parity = Parity.NONE
        self.lora.parity = expected_parity
        self.assertEqual(expected_parity, self.lora.parity)

    def test_air_speed(self):
        self.assertEqual(Mode.CONFIGURATION, self.lora.mode)
        expected_air_speed = AirSpeed.K4_8
        self.lora.air_speed = expected_air_speed
        self.assertEqual(expected_air_speed, self.lora.air_speed)

    def test_packet_size(self):
        self.assertEqual(Mode.CONFIGURATION, self.lora.mode)
        expected_packet_size = PacketSize.SIZE_128
        self.lora.packet_size = expected_packet_size
        self.assertEqual(expected_packet_size, self.lora.packet_size)

    def test_ambient_noise(self):
        self.assertEqual(Mode.CONFIGURATION, self.lora.mode)
        expected_ambient_noise = AmbientNoise.ENABLED
        self.lora.ambient_noise = expected_ambient_noise
        self.assertEqual(expected_ambient_noise, self.lora.ambient_noise)

    def test_transmit_power(self):
        self.assertEqual(Mode.CONFIGURATION, self.lora.mode)
        expected_transmit_power = TransmitPower.DBM_13
        self.lora.transmit_power = expected_transmit_power
        self.assertEqual(expected_transmit_power, self.lora.transmit_power)

    def test_rssi(self):
        self.assertEqual(Mode.CONFIGURATION, self.lora.mode)
        expected_rssi = RSSI.ENABLED
        self.lora.rssi = expected_rssi
        self.assertEqual(expected_rssi, self.lora.rssi)

    def test_transfer_method(self):
        self.assertEqual(Mode.CONFIGURATION, self.lora.mode)
        expected_transfer_method = TransferMethod.FIXED
        self.lora.transfer_method = expected_transfer_method
        self.assertEqual(expected_transfer_method, self.lora.transfer_method)

    def test_relay(self):
        self.assertEqual(Mode.CONFIGURATION, self.lora.mode)
        expected_relay = Relay.ENABLED
        self.lora.relay = expected_relay
        self.assertEqual(expected_relay, self.lora.relay)

    def test_lbt(self):
        self.assertEqual(Mode.CONFIGURATION, self.lora.mode)
        expected_lbt = LBT.ENABLED
        self.lora.lbt = expected_lbt
        self.assertEqual(expected_lbt, self.lora.lbt)

    def test_wor_control(self):
        self.assertEqual(Mode.CONFIGURATION, self.lora.mode)
        expected_wor_control = WORControl.RECEIVE
        self.lora.wor_control = expected_wor_control
        self.assertEqual(expected_wor_control, self.lora.wor_control)

    def test_wor_period(self):
        self.assertEqual(Mode.CONFIGURATION, self.lora.mode)
        expected_wor_period = WORPeriod.MS_500
        self.lora.wor_period = expected_wor_period
        self.assertEqual(expected_wor_period, self.lora.wor_period)

    def test_crypt_key_h(self):
        self.assertEqual(Mode.CONFIGURATION, self.lora.mode)
        self.lora.crypt_key_h = 222
        self.assertEqual(222, self.lora.crypt_key_h)

    def test_crypt_key_l(self):
        self.assertEqual(Mode.CONFIGURATION, self.lora.mode)
        self.lora.crypt_key_l = 111
        self.assertEqual(111, self.lora.crypt_key_l)

    def test_crypt_key(self):
        self.assertEqual(Mode.CONFIGURATION, self.lora.mode)
        expected_crypt_key = CryptKey(100, 200)
        self.lora.crypt_key = expected_crypt_key
        self.assertEqual(expected_crypt_key.hi, self.lora.crypt_key_h)
        self.assertEqual(expected_crypt_key.lo, self.lora.crypt_key_l)
        self.assertEqual(expected_crypt_key, self.lora.crypt_key)



from enum import IntEnum


class BaudRate(IntEnum):
    B1200 = 1200
    B2400 = 2400
    B4800 = 4800
    B9600 = 9600
    B19200 = 19200
    B38400 = 38400
    B57600 = 57600
    B115200 = 115200

    @staticmethod
    def from_data(data: int) -> "BaudRate":
        bits = (data & 0b11100000) >> 5
        bit_to_baud = {
            0b000: BaudRate.B1200,
            0b001: BaudRate.B2400,
            0b010: BaudRate.B4800,
            0b011: BaudRate.B9600,
            0b100: BaudRate.B19200,
            0b101: BaudRate.B38400,
            0b110: BaudRate.B57600,
            0b111: BaudRate.B115200,
        }
        try:
            return bit_to_baud[bits]
        except KeyError:
            raise ValueError(f"Invalid baud rate bits: {bits:03b}")

    def apply_to(self, data: int) -> int:
        baud_to_bits = {
            BaudRate.B1200: 0b000,
            BaudRate.B2400: 0b001,
            BaudRate.B4800: 0b010,
            BaudRate.B9600: 0b011,
            BaudRate.B19200: 0b100,
            BaudRate.B38400: 0b101,
            BaudRate.B57600: 0b110,
            BaudRate.B115200: 0b111,
        }
        bit_value = baud_to_bits[self] << 5
        cleared = data & 0b00011111
        return cleared | bit_value
